# source: pages 8 and 16 on PDF:
# https://global.download.synology.com/download/Document/Software/DeveloperGuide/Os/DSM/All/enu/DSM_Login_Web_API_Guide_enu.pdf

# NOTE: https://global.synologydownload.com/download/Document/Software/DeveloperGuide/Package/Calendar/2.4/enu
# /Synology_Calendar_API_Guide_enu.pdf, Refers to common error code # 160, "Insufficient application privilege" Page 10.
from __future__ import annotations

CODE_SUCCESS = 0
CODE_UNKNOWN = 9999
# 'Common' Error Codes:
error_codes = {
    CODE_SUCCESS: 'Success',
    100: 'Unknown error',
    101: 'No parameter of API, method or version',
    102: 'The requested API does not exist',
    103: 'The requested method does not exist',
    104: 'The requested version does not support the functionality',
    105: 'The logged in session does not have permission',
    106: 'Session timeout',
    107: 'Session interrupted by duplicated login',
    108: 'Failed to upload the file',
    109: 'The network connection is unstable or the system is busy',
    110: 'The network connection is unstable or the system is busy',
    111: 'The network connection is unstable or the system is busy',
    112: 'Preserve for other purpose',
    113: 'Preserve for other purpose',
    114: 'Lost parameters for this API',
    115: 'Not allowed to upload a file',
    116: 'Not allowed to perform for a demo site',
    117: 'The network connection is unstable or the system is busy',
    118: 'The network connection is unstable or the system is busy',
    119: 'Invalid session / SID not found.',
    # 120-149 Preserve for other purpose
    120: 'Preserve for other purpose',
    121: 'Preserve for other purpose',
    122: 'Preserve for other purpose',
    123: 'Preserve for other purpose',
    124: 'Preserve for other purpose',
    125: 'Preserve for other purpose',
    126: 'Preserve for other purpose',
    127: 'Preserve for other purpose',
    128: 'Preserve for other purpose',
    129: 'Preserve for other purpose',
    130: 'Preserve for other purpose',
    131: 'Preserve for other purpose',
    132: 'Preserve for other purpose',
    133: 'Preserve for other purpose',
    134: 'Preserve for other purpose',
    135: 'Preserve for other purpose',
    136: 'Preserve for other purpose',
    137: 'Preserve for other purpose',
    138: 'Preserve for other purpose',
    139: 'Preserve for other purpose',
    140: 'Preserve for other purpose',
    141: 'Preserve for other purpose',
    142: 'Preserve for other purpose',
    143: 'Preserve for other purpose',
    144: 'Preserve for other purpose',
    145: 'Preserve for other purpose',
    146: 'Preserve for other purpose',
    147: 'Preserve for other purpose',
    148: 'Preserve for other purpose',
    149: 'Preserve for other purpose',
    150: 'Request source IP does not match the login IP',
    160: 'Insufficient application privilege',
    CODE_UNKNOWN: 'Unknown Error',
}
# Source: https://global.synologydownload.com/download/Document/Software/DeveloperGuide/Os/DSM/All/enu
# /DSM_Login_Web_API_Guide_enu.pdf Page 16.
# https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/SurveillanceStation/All/enu
# /Surveillance_Station_Web_API.pdf Pages 32,33 Refers to Auth error code #411: 'Account Locked (when account max try
# exceed).'
auth_error_codes: dict[int, str] = {
    400: 'No such account or incorrect password',
    401: 'Disabled account',
    402: 'Denied permission',
    403: '2 - factor authentication code required',
    404: 'Failed to authenticate 2 - factor authentication code',
    406: 'Enforce to authenticate with 2 - factor authentication code',
    407: 'Blocked IP source',
    408: 'Expired password cannot change',
    409: 'Expired password',
    410: 'Password must be changed',
    411: 'Account Locked (when account max try exceed).',
}

# Source:https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/DownloadStation/All/enu/Synology_Download_Station_Web_API.pdf
# Page 29
download_station_error_codes: dict[int, str] = {
    400: 'File upload failed',
    401: 'Max number of tasks reached',
    402: 'Destination denied',
    403: 'Destination does not exist',
    404: 'Invalid task id',
    405: 'Invalid task action',
    406: 'No default destination',
    407: 'Set destination failed',
    408: 'File does not exist',
}

# TODO use the error code source https://cndl.synology.cn/download/Document/Software/DeveloperGuide/Package
#  /FileStation/All/enu/Synology_File_Station_API_Guide.pdf page 10~11
file_station_error_codes: dict[int, str] = {
    400: 'Invalid parameter of file operation',
    401: 'Unknown error of file operation',
    402: 'System is too busy',
    403: 'Invalid user does this file operation',
    404: 'Invalid group does this file operation',
    405: 'Invalid user and group does this file operation',
    406: "Can't get user/group information from the account server",
    407: 'Operation not permitted',
    408: 'No such file or directory',
    409: 'Non-supported file system',
    410: 'Failed to connect internet-based file system (e.g., CIFS)',
    411: 'Read-only file system',
    412: 'Filename too long in the non-encrypted file system',
    413: 'Filename too long in the encrypted file system',
    414: 'File already exists',
    415: 'Disk quota exceeded',
    416: 'No space left on device',
    417: 'Input/output error',
    418: 'Illegal name or path',
    419: 'Illegal file name',
    420: 'Illegal file name on FAT file system',
    421: 'Device or resource busy',
    599: 'No such task of the file operation',
}

# Source: https://global.synologydownload.com/download/Document/Software/DeveloperGuide/Package/Virtualization/All
# /enu/Synology_Virtual_Machine_Manager_API_Guide.pdf Page 8,9
virtualization_error_codes: dict[int, str] = {
    401: 'Bad parameter.',
    402: 'Operation failed.',
    403: 'Name conflict.',
    404: 'The number of iSCSI LUNs has reached the system limit. Note: vdisk is based on iSCSI LUN, which is also '
         'limited by the system.',
    500: 'The cluster is frozen. More than half of the hosts are offline.',
    501: 'The cluster is in the incompatible mode. Please upgrade to a compatible DSM version and try again.',
    600: 'The cluster is not ready.',
    601: 'The host is offline.',
    700: 'The storage is in invalid.',
    900: 'Failed to set a host to a virtual machine.',
    901: 'The virtual machine does not have a host.',
    902: 'Failed to power on a virtual machine due to insufficient CPU threads.',
    903: 'Failed to power on a virtual machine due to insufficient memory.',
    904: 'The status of virtual machine is online.',
    905: 'MAC conflict.',
    906: 'Failed to create virtual machine because the selected image is not found.',
    907: 'The status of virtual machine is offline.',
    908: 'Failed to power on a virtual machine due to insufficient CPU threads for reservation on the host.',
    909: 'Failed to power on the virtual machine because there is no corresponding networking on the host.',
    910: 'Only the VirtIO hard disk controller can be used to boot the virtual machine remotely.',
    911: 'Virtual machines with UEFI enabled cannot be powered on remotely.',
    1000: 'Cannot find task_id.',
    1001: 'Need Virtual Machine Manager Pro.',
    1400: 'The result of image creating is partial success.',
    1600: 'The virtual machine has been successfully edited. However, errors occurred while reserving the memory or '
          'CPU on the HA hosts.',
}

# Source: https://global.synologydownload.com/download/Document/Software/DeveloperGuide/Package/Calendar/2.4/enu
# /Synology_Calendar_API_Guide_enu.pdf Pages 10,11.
calendar_error_codes: dict[int, str] = {
    400: 'Invalid parameter of file operation',
    401: 'Unknown error of file operation',
    402: 'System is too busy',
    403: 'This user does not have permission to execute this operation',
    404: 'This group does not have permission to execute this operation',
    405: 'This user/group does not have permission to execute this operation',
    406: 'Cannot obtain user/group information from the account server',
    407: 'Operation not permitted',
    408: 'No such file or directory',
    409: 'File system not supported',
    410: 'Failed to connect internet-based file system (ex: CIFS)',
    411: 'Read-only file system',
    412: 'Filename too long in the non-encrypted file system',
    413: 'Filename too long in the encrypted file system',
    414: 'File already exists',
    415: 'Disk quota exceeded',
    416: 'No space left on device',
    417: 'Input/output error',
    418: 'Illegal name or path',
    419: 'Illegal file name',
    420: 'Illegal file name on FAT file system',
    421: 'Device or resource busy',
    599: 'No such task of the file operation',
}

# # Source: https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package
# /SurveillanceStation/All/enu/Surveillance_Station_Web_API.pdf NOTE: Pages 81, 131,132, 155, 190, 222, 305, 314,
# 321, 328, 451, 473 539, 556, contain unique items. Most of these are duplicate information, but they describe
# different methods, all the error codes I found are on # Pages: 71, 81, 85, 93, 103, 113, 131, 132, 139, 144, 155,
# 167, 169, 176, 187, 190, 191, 201, 211, 212, 217, 222, 227, #         241, 245, 249, 253, 264, 279, 281, 305, 314,
# 321, 328, 363, 365, 368, 369, 393, 395, 397, 403, 410, 412, 415, #         419, 430, 451, 473, 539, 556
surveillance_station_error_codes: dict[int, str] = {
    400: 'Execution failed.',
    401: 'Parameter invalid.',
    402: 'Camera disabled.',
    403: 'Insufficient license.',
    404: 'Codec activation failed',
    405: 'CMS server connection failed.',
    407: 'CMS closed.',
    412: 'Need to add license.',
    413: 'Reach the maximum of platform',
    414: 'Some events not exist.',
    415: 'message connect failed.',
    417: 'Test Connection Error.',
    418: 'Object is not exist. / The VisualStation ID does not exist.',
    419: 'Visualstation name repetition.',
    439: 'Too many items selected',
    446: 'Task Path already exist.',
    522: 'Original Task is Migrating',
    534: 'Exceed name length limitation.',
    543: 'The number of DVA tasks and face tasks exceed the limitation',
    548: 'The input video type of the DVA task is invalid',
    553: 'No face detected.',
    554: 'Face detected is too small.',
    555: 'Multiple faces detected.',
    556: 'The account try to set has exists.',
    557: 'No image data appended.',
    558: 'Add face group failed.',
    560: 'The camera try to apply to face task is occupied by another face task.',
    561: 'The number of face profile exceeds the maximum limit.',
    562: 'The number of face group exceeds the maximum limit.',
    563: 'The face profile is created or edited failed due to duplicated account.',
    564: 'The face result try to query doesn\'t exist',
    567: 'The face database is under synchronization.',
}

# # Source: sds.bundle.js from DSM
core_error_codes: dict[int, str] = error_codes
core_error_codes.update({
    1101: "Failed to apply the settings.",
    1102: "Failed to restore firewall rules.",
    1103: "Your computer has been blocked by the new firewall configuration. The firewall configuration has been reset to the previous state. Please make sure that no rule is blocking your computer and try again.",
    1104: "The maximum number of firewall rules has already been reached.",
    1105: "Failed to disable the firewall rule. Please go to Control Panel > Security > Firewall to manage rules.",
    1198: "Operation incompatible with this version.",
    1201: "Failed to apply the settings.",
    1202: "The sum of the maximum bandwidth is too large. Please enter lower values.",
    1203: "The largest maximum bandwidth you entered is too large. Please enter a lower one.",
    1204: "Failed to restore Traffic Control rules.",
    1301: "Failed to apply the settings.",
    1302: "Failed to restore DoS protection settings.",
    1402: "Failed to load domain",
    1410: "Unable to perform this operation. Please contact <a class=\"link-font\" target=\"_blank\" href=\"https://www.synology.com/company/contact_us\">Synology Technical Support</a>.",
    1500: "The operation failed. Please sign in to DSM again and retry.",
    1501: "Another user with administrative privilege is configuring the setting of this page. Please try again later.",
    1502: "Failed to retrieve external IP address. Please check your router and DNS settings.",
    1503: "Your gateway setting is invalid.",
    1504: "Your DNS server settings might be incorrect or not working correctly. Please check the settings at Control Panel > Network.",
    1510: "Failed to update the router database.",
    1521: "This router supports up to {0} ports to be assigned for port forwarding. Please remove some rules and try again.",
    1522: "This router supports up to {0} port forwarding rules. Please remove some rules and try again.",
    1523: "This router supports up to {0} single ports and {1} port ranges to be assigned for port forwarding. Please remove some rules and try again.",
    1524: "The maximum number ({0}) of rules allowed for the router has been exceeded. Please remove some rules and try again.",
    1525: "The following router port numbers conflict",
    1526: "Failed to add the following router port numbers",
    1527: "Failed to apply the port forwarding rules to the router.",
    1528: "Please enable the UPnP or NAT-PMP protocol on the router first.",
    1530: "This router is not supported by the current DSM on your Synology NAS. Please update the DSM and try again.",
    1600: "Unable to perform this operation. Please check if your network connection, power supply, and UPS are working properly, and try again.",
    1601: "Failed to configure UPS settings. Please check whether your network connection, power supply, and UPS are working properly.",
    1602: "Failed to obtain UPS information. Please check whether your network connection, power supply, and UPS are working properly.",
    1701: "One of the port numbers is used by another service. Please choose another port range.",
    1702: "The name is taken. Please choose a different name.",
    1703: "File or directory does not exist.",
    1704: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    1706: "Volume of the destination is read-only. The volume might be crashed. Please contact the system administrator.",
    1903: "One of the port numbers is used by another service. Please choose another port range.",
    1904: "One of the port numbers is used by another service. Please choose another port range.",
    1905: "The anonymous FTP/WebDAV user no longer has access permissions for the shared folder.",
    1951: "One of the port numbers is used by another service. Please choose another port range.",
    2001: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2002: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2101: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2102: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2201: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2202: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2301: "Unable to perform this operation. Please try again later.",
    2303: "One of the port numbers is used by another service. Please choose another port range.",
    2331: "Unsupported key format",
    2332: "Failed to upload the file. Please check your network connection.",
    2371: "Failed to set the NFS permissions for the shared folder(s). Please unmount all remote folder(s) within the shared folder(s) and try again. ",
    2372: "You cannot assign NFS permissions for an HFS+ file system.",
    2401: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2402: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2403: "One of the port numbers is used by another service. Please choose another port range.",
    2500: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2502: "Unable to perform this operation. Please try again later.",
    2503: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2504: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2505: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2601: "Invalid domain name. Please check the DNS setting of the domain server and use complete domain name to join.",
    2602: "Failed to join domain. Please check the DNS server settings and make sure the domain is registered on the DNS server.",
    2603: "Invalid DC IP.",
    2604: "The workgroup name is invalid or contains invalid characters.",
    2605: "Unable to join the domain because the domain controller cannot be detected.",
    2606: "Failed to connect to the domain controller (DC) via TCP port 445. Please check the network and firewall settings of the DC.",
    2607: "Invalid user name or password.",
    2608: "This account has been disabled or expired, or is not allowed to logon now. Please contact the domain administrator.",
    2609: "This account has been disabled. Please contact the domain administrator.",
    2610: "Invalid user name or password, or the account has expired. Please contact the domain administrator.",
    2611: "Permission denied. Please use an account that belongs to the administrators group on the domain to join.",
    2612: "The account password needs to be changed before logon. Please contact the domain administrator.",
    2613: "Cannot find the domain workgroup. Please enter a correct DC IP.",
    2614: "This account is not allowed to logon now, or cannot logon to the workstation, or the account password needs to be changed before logon. Please contact the domain administrator.",
    2615: "Unable to join the domain. Please check your domain and firewall settings and try again.",
    2616: "You need to enable \"Windows File Service\" before joining a domain. Do you want to enable the service now?",
    2626: "Updating the domain user/group list. The time required depends on the object count to be synced from the domain server to the client. Domain operations will be available once the update is complete.",
    2628: "Failed to join the domain because Synology Directory Server already provides domain service.",
    2702: "Connected",
    2703: "Disconnected",
    2704: "Another administrator is configuring in this page. Please try again later.",
    2705: "The operation failed. Please sign in to DSM again and retry.",
    2706: "Invalid credentials. Please check your account name and password.",
    2707: "Failed to connect to the LDAP server. Please check the server address and your network settings.",
    2708: "The LDAP server is not supported.",
    2709: "This Synology NAS cannot be joined to any LDAP directory because it is already a domain client.",
    2710: "Failed to connect to the LDAP server. Please check the server address and your network settings.",
    2712: "{0} doesn't exist on the LDAP server. It may be removed by the administrator. Please refresh the page and try again later.",
    2713: "Failed to connect to the LDAP server. Please try again later.",
    2714: "Invalid DN format.",
    2715: "Permission denied. Please contact the server administrator.",
    2716: "Permission denied. Please contact the server administrator.",
    2717: "The operation timed out. Please contact the administrator of the LDAP server.",
    2718: "LDAP server does not allow anonymous binding. Please check your base DN, bind DN, and password.",
    2719: "When the CIFS-related option is enabled, the system will set the \"Maximum SMB protocol\" to SMB 1 and disable the SMB-related Bonjour service (at Control Panel > File Services > Service Discovery).",
    2721: "Encrypted connection is required by the LDAP server.",
    2723: "This password may be too short, too weak, too common, or has been used before. Please enter another one.",
    2799: "The operation failed. Please sign in to DSM again and retry.",
    2800: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2801: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2900: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2901: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    2902: "Network error occurred. Please check your network settings.",
    2903: "Oops... An error occurred with the QuickConnect server. Please try again later.",
    2904: "{0} is already in use.",
    2905: "Wrong Synology Account username or password.",
    2906: "This QuickConnect ID is currently used by another Synology product.",
    3000: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    3001: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    3002: "Network error occurred. Please check your DNS and network settings.",
    3003: "Oops... An error occurred with the Synology Account server. Please try again later.",
    3004: "Authorization Required",
    3005: "{0} is already in use.",
    3006: "This Synology NAS has exceeded its daily number of new Synology Accounts. Please try again tomorrow.",
    3009: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    3010: "Already logged in as \"{0}\"",
    3013: "An error has occurred when you try to log in. The migration wizard will be terminated. To enable the service, please log in at Synology Account manually or contact {0}Synology Support{1} for technical assistance.",
    3015: "Error Code: 3015. Please contact Synology Support for technical assistance.",
    3106: "The user doesn't exist.",
    3107: "The user name already exists or is reserved for system use only. Please enter a different one.",
    3108: "The user name already exists or is reserved for system use only. Please enter a different one.",
    3109: "Failed to deactivate the account. There should be at least one valid administrator in the system.",
    3110: "You have reached the maximum limit of {0} user accounts",
    3111: "User home share folder is missing, please restart the service.",
    3112: "Another user with administrative privilege is configuring the setting of this page. Please try again later.",
    3113: "Another administrator is configuring in this page. Please try again later.",
    3114: "The user name already exists or is reserved for system use only. Please enter a different one.",
    3115: "The system default account cannot be deleted.",
    3116: "Failed to setup user accounts",
    3117: "The user quota exceeds the free space available on the volume.",
    3118: "Operation failed because the available volume size is insufficient.",
    3119: "The user homes folder is being moved. Please wait for the folder to finish moving and try again.",
    3121: "The password is invalid.",
    3122: "The password you set must be different from the previously used ones.",
    3123: "The password you set is not strong enough.",
    3124: "The password is invalid.",
    3130: "Invalid syntax in file. Extra text found after double quotation marks. Please check the below field:<br>{0}",
    3131: "Invalid syntax in file. Double quotation marks placed incorrectly. Please check the below field:<br>{0}",
    3132: "Invalid syntax in file. Missing double quotation marks at the end of a field. Please check the below field:<br>{0}",
    3191: "Failed to open the file. Please check if the file is damaged.",
    3192: "The list is empty.",
    3193: "File encoding must be saved as UTF-8.",
    3194: "There is no volume in your system. Please go to <b>Storage Manager</b> and create a new volume.",
    3202: "Another administrator is configuring in this page. Please try again later.",
    3204: "Failed to load group data. Please try again later.",
    3205: "Failed to load group data. Please try again later.",
    3206: "The group name already exists or is reserved for system use only. Please enter a different one.",
    3207: "The group name already exists or is reserved for system use only. Please enter a different one.",
    3208: "The group name is invalid or contains invalid characters.",
    3209: "The number of user group cannot exceed {0}.",
    3210: "Failed to remove the member(s) from the administrators group. There should be at least one valid administrator in the system.",
    3217: "Reached the maximum limit of domain administrators groups.",
    3221: "The number of Windows ACL rules for this shared folder exceeds the system limit.(Maximum number of entries: 200)",
    3299: "The operation failed. Please sign in to DSM again and retry.",
    3301: "A shared folder with the same name already exists. Please enter another name.",
    3302: "The volume where the share is located does not support ACL. Please refer to Help for more information.",
    3303: "You can only encrypt a non-system reserved shared folder",
    3304: "Volume not found. Please check if the volume status is normal.",
    3305: "The shared folder name is invalid or it contains invalid characters.",
    3308: "Invalid key. Please enter or import a valid key for decryption.",
    3309: "The maximum number of shared folders is {0}.",
    3312: "A folder with the same name already exists on this volume. Please enter a different one.",
    3313: "Volume not found. Please check if the volume status is normal.",
    3314: "This volume is currently read-only.",
    3319: "The shared folder name is either reserved for system use only or already existed. Please enter a different name.",
    3320: "There is not enough free space on the volume.",
    3321: "The number of Windows ACL rules for this shared folder exceeds the system limit.(Maximum number of entries: 200)",
    3322: "Something went wrong when mounting the shared folder. Please contact Synology support.",
    3323: "Failed to change the shared folder's location. Please unmount the virtual drive(s) or remote folder(s) within the shared folder and try again.",
    3324: "Failed to rename the shared folder. Please unmount the virtual drive(s) or remote folder(s) within the shared folder and try again.",
    3326: "The encryption key you imported is invalid. Please import a valid encryption key.",
    3327: "Unable to perform this operation. A folder on the target volume has the same name as the folder to be moved. Please change the name of the folder to be moved and try again.",
    3328: "This shared folder is locked by another process. Please try again later.",
    3329: "You have reached maximum limit of {0} retained snapshots for each shared folder, or {1} retained snapshots in total for the system.",
    3330: "The shared folder is being used by another process. Please try again later.",
    3332: "System is currently backing up or restoring. Please try again later.",
    3334: "Failed to restore the shared folder. Please unmount the virtual drive(s) or remote folder(s) within the shared folder and try again.",
    3335: "This shared folder is used for replication tasks, but [Snapshot Replication] feature is not supported by the destination volume.",
    3336: "Replication task(s) of shared folder are in progress. Please wait for the completion or stop the syncing by going to [Snapshot Replication].",
    3337: "Preserved by the system.",
    3338: "Please unmount the encrypted shared folder, and try again.",
    3340: "Failed to restore the settings of the shared folder. Please try again.",
    3341: "Operation failed because of reaching the quota limit of this shared folder.",
    3344: "Incorrect passphrase.",
    3345: "Your NAS server initiated a key store in this location previously. To avoid data conflict, please choose a USB storage device that does not contain a key store.",
    3347: "Failed to unmount the encrypted shared folder. Please check if it is currently being accessed.",
    3350: "Unable to perform this operation because some shared folders have immutable snapshots.",
    3400: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    3401: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    3402: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    3403: "The specified user or group was not found. Please make sure the entered information is correct and check your network status.",
    3404: "Sorry, this user account has not been assigned proper privileges to perform this action.",
    3405: "Unable to perform this operation due to invalid database format. Please restart your NAS and try again.",
    3500: "Unable to perform this operation. Please try again later.",
    3501: "The port number should be from 1 to 65535.",
    3502: "One of the port numbers is used by another service. Please choose another port range.",
    3503: "One of the port numbers is used by another service. Please choose another port range.",
    3504: "One of the port numbers is used by another service. Please choose another port range.",
    3505: "The domain name is already used. Please use another name.",
    3506: "\"Server\" header cannot contain \\. Please enter another header.",
    3510: "Unable to perform this operation. Please try again later.",
    3511: "This port number is used by the other protocol. Please enter a different number.",
    3550: "There is no volume in your system.",
    3551: "No shared folders",
    3552: "Cannot enable {0} due to crashed volume.Web Station",
    3553: "Expanding volume...",
    3554: "One of the port numbers is used by another service. Please choose another port range.",
    3555: "The port number should be from 1 to 65535.",
    3603: "No available volume. Please create a new volume.",
    3604: "The available drive space of this system is insufficient.",
    3605: "Fail to download and install the driver.",
    3606: "The account or password is invalid. Please try again.",
    3607: "The IP address you entered is invalid.",
    3608: "The IP address you entered is already used by another network printer. Please enter another one.",
    3609: "Failed to determine if the network printer could be used. Please try again later.",
    3610: "This function is not available in the demo mode.",
    3611: "The printer name you entered is reserved or already used. Please enter another name.",
    3700: "Unable to perform this operation. Please try again later.",
    3701: "Not available",
    3702: "Unable to perform this operation. Please try again later.",
    3710: "Not available",
    3711: "Unable to perform this operation. Please try again later.",
    3712: "Fan mode not supported on server model",
    3720: "Not available",
    3721: "Unable to perform this operation. Please try again later.",
    3730: "Not available",
    3731: "Unable to perform this operation. Please try again later.",
    3740: "Not available",
    3741: "Unable to perform this operation. Please try again later.",
    3750: "Not available",
    3751: "Unable to perform this operation. Please try again later.",
    3760: "Not available",
    3761: "Unable to perform this operation. Please try again later.",
    3795: "One of the port numbers is used by another service. Please choose another port range.",
    3800: "Unable to perform this operation. Please try again later.",
    3801: "Unable to perform this operation. Please try again later.",
    4000: "Unable to perform this operation. Please try again later.",
    4001: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4002: "Unsupported format.",
    4003: "File size exceeds the 1MB limit.",
    4005: "File size exceeds the 1MB limit.",
    4006: "File size exceeds the 25 MB limit.",
    402: "This shared folder doesn't exist.",
    403: "Unable to perform this operation. Please try again later.",
    404: "Sorry, this user account has not been assigned proper privileges to perform this action.",
    4100: "Unable to perform this operation. Please try again later.",
    4101: "Unable to perform this operation. Please try again later.",
    4102: "The alias is reserved for system use only. Please enter another alias.",
    4103: "This alias already exists. Please enter a different alias.",
    4104: "This port number is used by another application. Please enter a different number.",
    4105: "This port number is used by another application. Please enter a different number.",
    4106: "This port number is used by another application. Please enter a different number.",
    4107: "The domain name is already used. Please use another name.",
    4154: "The domain name is already used. Please use another name.",
    4155: "This port number is used by another application. Please enter a different number.",
    4156: "Destination device unreachable. Please enter another hostname and check the DNS settings.",
    4164: "Invalid header name",
    4165: "Invalid value",
    4166: "Header name duplicated",
    4168: "Setting value must be between 1-86400 seconds.",
    4169: "Setting value must be between 1-86400 seconds.",
    4170: "Setting value must be between 1-86400 seconds.",
    4300: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4301: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4302: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4303: "Unable to perform this operation. Please try again later.",
    4304: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4305: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4306: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4307: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4308: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4309: "Unable to perform this operation. Please try again later.",
    4310: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4311: "No such network interface.",
    4312: "The IP address has been used!",
    4313: "This IPv6 IP address has been used.",
    4314: "Connection failed. Please check your network settings or contact the service provider.",
    4315: "Unable to set link-local address. Please try a different one.",
    4316: "Applying network configuration, please wait.",
    4317: "The IP address and the default gateway belong to two different subnets.",
    4319: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4320: "The profile name is already in use. Please enter a different name.",
    4321: "Invalid certificate. Refer to {0}this article{1} for more information.",
    4322: "Invalid private key.",
    4323: "File encoding must be saved as UTF-8.",
    4324: "",
    4325: "Connection failed. Please go to Main Menu > VPN Server to disable L2TP/IPSec server and try again.",
    4326: "Connection failed. Please go to \"Main Menu\" > \"VPN Server\" to disable OpenVPN server and then try again.",
    4327: "The .ovpn file contains invalid parameters. Please import a different file.",
    4328: "OpenVPN cannot support private key with password",
    4340: "Processing",
    4350: "IPv6 config corrupted",
    4351: "",
    4352: "Failed to enable IPv6 router.",
    4353: "Failed to disable IPv6 router.",
    4354: "Creating an IPv6 tunnel requires an IPv4 public address.",
    4370: "{0} is not supported when any bond exists. Please remove the bond and try again.",
    4371: "{0} is not supported if any interface has VLAN tagging enabled. Please remove the VLAN tagging and try again.",
    4372: "{0} is not supported in Wireless AP mode. Please change to other modes and try again.",
    4373: "The configuration of Open vSwitch has been changed. Please try again.",
    4380: "Ping target is invalid or network is unreachable.",
    4381: "Ping has been terminated due to timeout to save CPU resources.",
    4382: "Traceroute target is invalid or network is unreachable.",
    4500: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4501: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4502: "The installed volume is not ready.",
    4503: "Volume is being created in your system. Click \"OK\" to launch \"Storage Manager\" and check volume status.",
    4504: "The free space of system partition is insufficient. Please contact the Synology support team for help.",
    4506: "Oops, installation cannot be canceled now.",
    4520: "The available drive space of this system is insufficient.",
    4521: "Invalid file format.",
    4522: "The package file is corrupt. Please download it again or contact the package developer.",
    4529: "To update this application, please first uninstall the existing version.",
    4530: "Another package is being configured, installed, or restored. Please try again later.",
    4531: "This package is not published by Synology Inc.",
    4532: "This package is published by an unknown publisher.",
    4533: "This package's digital certificate has already expired.",
    4534: "",
    4535: "The package file is corrupt. Please download it again or contact the package developer.",
    4540: "Failed to install the package.",
    4541: "Failed to update.",
    4542: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4543: "Invalid file format.",
    4544: "This package has already been installed.",
    4545: "Not available",
    4548: "The version of the package installed is {0}, which is not compatible with your Synology NAS. Please update it to {1} or above.",
    4549: "A package conflict occurred. Please contact the developer of this package.",
    4570: "The serial number of your Synology NAS is either incorrect or empty.",
    4580: "Failed to run the package service.",
    4581: "Failed to stop the package service.",
    4590: "Invalid location.",
    4591: "The name or location is already in use.",
    4592: "This certificate has already been imported.",
    4593: "This certificate is already trusted by Synology Inc.",
    4594: "This certificate has already been revoked.",
    4595: "Invalid certificate",
    4600: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4601: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4602: "Gmail OAuth refresh token error. Please check your network connection or sign in to your Google account again.",
    4603: "Outlook OAuth refresh token error. Please check your network connection or sign in to your Outlook account again.",
    4631: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4632: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4633: "This SMS service provider cannot be found. Please choose a different one and try again.",
    4634: "This SMS service provider already exists. Please choose a different one.",
    4635: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4661: "Failed to forward Synology NAS information to push service server.",
    4681: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4682: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4683: "This webhook provider cannot be found. Please choose a different one.",
    4684: "This webhook provider already exists. Please choose a different one.",
    4685: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4800: "Unable to perform task. Please try again later or {0}contact us{1} for technical support if the issue persists.",
    4801: "Unable to load task. Please try again later or {0}contact us{1} for technical support if the issue persists.",
    4802: "Unable to delete task. Please try again later or {0}contact us{1} for technical support if the issue persists.",
    4803: "Unable to run task. Please try again later or {0}contact us{1} for technical support if the issue persists.",
    4804: "Unable to save task. Please try again later or {0}contact us{1} for technical support if the issue persists.",
    4900: "Unable to perform this operation. Please try again later.",
    4901: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    4902: "The user doesn't exist.",
    4903: "The path to save reports does not exist.",
    4904: "The name is taken. Please choose a different name.",
    4905: "The available drive space of this system is insufficient.",
    5000: "Unable to perform this operation. Please try again later.",
    5001: "Unable to perform this operation. Please try again later.",
    5002: "Unable to perform this operation. Please try again later.",
    5003: "Unable to perform this operation. Please try again later.",
    5004: "Unable to perform this operation. Please try again later.",
    5005: "",
    5006: "",
    5007: "",
    5008: "No active logs",
    5009: "One of the port numbers is used by another service. Please choose another port range.",
    5010: "Unable to perform this operation. Please try again later.",
    5011: "Unable to perform this operation. Please try again later.",
    5012: "",
    5016: "Unable to load system settings, possibly because the network connection is unstable or the system is busy. Please try again later.<a href=\"https://www.synology.com/company/contact_us\" target=\"_blank\"></a>",
    5017: "Unable to load system settings, possibly because the network connection is unstable or the system is busy. Please try again later.<a href=\"https://www.synology.com/company/contact_us\" target=\"_blank\"></a>",
    5018: "Unable to load settings, possibly because the network connection is unstable or the system is busy. Please try again later.<a href=\"https://www.synology.com/company/contact_us\" target=\"_blank\"></a>",
    5019: "Unable to load system settings, possibly because the network connection is unstable or the system is busy. Please try again later.<a href=\"https://www.synology.com/company/contact_us\" target=\"_blank\"></a>",
    5020: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.<a href=\"https://www.synology.com/company/contact_us\" target=\"_blank\"></a>",
    5021: "Unable to load system settings, possibly because the network connection is unstable or the system is busy. Please try again later.<a href=\"https://www.synology.com/company/contact_us\" target=\"_blank\"></a>",
    5022: "Operation failed because of reaching the quota limit of this shared folder.",
    5023: "The available drive space of this system is insufficient.",
    5024: "Unable to display logs due to insufficient free space. Refer to <a class=\"link-font\" target=\"_blank\" rel=\"noreferrer noopener\" href=\"https://sy.to/f543d\">this article</a> for more information.",
    5100: "Unable to perform this operation. Please try again later.",
    5101: "Unable to perform this operation. Please try again later.",
    5102: "Unable to perform this operation. Please try again later.",
    5103: "Unable to perform this operation. Please try again later.",
    5104: "Unable to perform this operation. Please try again later.",
    5105: "Unable to perform this operation. Please try again later.",
    5106: "Unable to perform this operation. Please try again later.",
    5202: "Failed to apply your settings because another installation is in progress.",
    5203: "Volume is busy. Please try again later.",
    5205: "The update you are applying is not compatible with this version of DSM. Please get the compatible file and try again.",
    5206: "DSM cannot be rolled back to an earlier version. The applied DSM must be newer than the current one.",
    5207: "The model of the update file is incompatible with the system model.",
    5208: "Failed to apply your settings because another installation is in progress.",
    5209: "Fail to update the file. The file is probably corrupt.",
    5210: "Cannot update DSM on the passive server. Please try again later.",
    5211: "The available disk space of the system is insufficient.",
    5213: "Another package is being configured, installed, or restored. Please try again later.",
    5214: "Connection failed. Please check your Internet connection.",
    5215: "The available drive space of this system is insufficient.",
    5216: "The selected folder is read-only.",
    5217: "Destination folder does not exist.",
    5219: "DSMupdate installation is in progress and cannot be cancelled.",
    5220: "No DSM update installation is running right now.",
    5221: "Failed to cancel the installation.",
    5222: "This file may have been corrupted. Please redownload the update file (.pat) from Synology Download Center.",
    5223: "Your system is not compatible with the patch file. Please contact Synology Support for assistance.",
    5224: "The file version of the active server is inconsistent with that of the passive server. Please make sure the file version is consistent",
    5225: "This file is incompatible with the passive server. Please select a file compatible with your model.",
    5300: "Unable to perform this operation. Please try again later.",
    5301: "The user doesn't exist.",
    5510: "Invalid certificate",
    5511: "Invalid private key.",
    5512: "Invalid intermediate certificate. Please import the correct certificate.",
    5513: "Invalid cipher type. Please use another certificate and private key.",
    5514: "Private key and certificate are not matched.",
    5515: "Please import your certificate issued by a trusted authority and try again.",
    5516: "File encoding must be saved as UTF-8.",
    5517: "Failed to verify the server and intermediate certificates. Please try again.",
    5518: "DSM does not support DSA certificates.",
    5519: "Invalid certificate signing request. Please select and upload a valid CSR file.",
    5520: "No response from the destination server. Please try again later.",
    5521: "Let's Encrypt is unable to validate this domain name. Please make sure your Synology NAS and router have port 80 open to Let's Encrypt domain validation from the Internet. All the other communications with Let's Encrypt go over HTTPS to keep your Synology NAS secure.",
    5522: "The certificate issuer is unable to validate this domain name. Please make sure the domain name is valid.",
    5523: "Maximal registered email accounts reached for this IP address.",
    5524: "Maximal certificate requests reached for this domain name.",
    5525: "Invalid email address.",
    5526: "Value of parameter is invalid.",
    5527: "The certificate server is busy. Please try again later.",
    5528: "Only domains of Synology DDNS support wildcard certificates issued by Let's Encrypt.",
    5529: "Invalid domain. Please make sure this domain can be resolved into a public IP address.",
    5530: "Please check if your IP address, reverse proxy rules, and firewall settings are correctly configured and try again.",
    5531: "Invalid real-name certification",
    5532: "Free certificate quota exceeded",
    5533: "Invalid access token",
    5534: "Invalid certificate. The certificate key should be more than 1024 bits in length.",
    5600: "File or directory does not exist.",
    5601: "An error occurred during file transfer. Please re-open the text file to display the complete content.",
    5602: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    5603: "Failed to load \"{0}\".",
    5604: "Failed to save the file.",
    5605: "Sorry, this user account has not been assigned proper privileges to perform this action.",
    5606: "Failed to load \"{0}\". Please try a different encoding.",
    5607: "This document contains characters that will not be saved correctly with the selected encoding. To keep the encoding information, please click No and resave the file using the correct encoding. Do you wish to save it anyway?",
    5608: "The name of the encrypted file or folder cannot exceed 143 English characters or 47 Asian (CJK) characters.",
    5609: "File/folder name should be within 255 characters.",
    5610: "Your personal data has exceeded the quota.",
    5611: "The available drive space of this system is insufficient.",
    5612: "I/O error occurred. Please try again later.",
    5613: "Sorry, this user account has not been assigned proper privileges to perform this action.",
    5614: "The selected folder is read-only.",
    5615: "The name is taken. Please choose a different name.",
    5616: "File or directory does not exist.",
    5617: "Destination folder does not exist.",
    5618: "Cannot find the specified domain. Please contact the system administrator.",
    5619: "File and folder names cannot contain colons (:) and slashes (/), start with ._ (e.g. ._name), or use any combination of characters that are reserved for system use (e.g. . or ..). Please enter another name.",
    5620: "File/folder name cannot contain \\/:*?\"<>|. Please choose a different name.",
    5621: "The file exceeds the maximum 10-MB file limit for Text Editor.",
    5623: "Cannot save the files. One or more files are protected by WriteOnce. To save other files, close the protected files.",
    5703: "The NTP service is required for Surveillance Station and High Availability cluster and therefore cannot be disabled.",
    5800: "Unable to perform this operation. Please try again later.",
    5801: "This shared folder doesn't exist.",
    5901: "Failed to apply the settings.",
    5902: "Failed to restore VPN pass-through settings.",
    5903: "VPN pass-through is only available on certain models.",
    6000: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    6001: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    6002: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    6003: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    6004: "Failed to load system settings.",
    6005: "Failed to apply the settings.",
    6006: "Failed to start service. Please check system log.",
    6007: "Failed to stop service. Please check system log.",
    6008: "Failed to start service. Please check system log.",
    6009: "Failed to save firewall rules.",
    6010: "The IP address you entered is invalid.",
    6011: "The IP address you entered is invalid.",
    6012: "The IP address you entered is invalid.",
    6013: "This shared folder doesn't exist.",
    6014: "There is no volume in the managed server. Please sign in to the managed server and create a volume.",
    6015: "No permissions to restart the TFTP service.",
    6016: "Invalid file path to root folder.",
    6200: "Unable to perform this operation, possibly because the network connection is unstable or the system is busy. Please try again later.",
    6201: "The volume where the shared folder is located does not support ACL. Please refer to Help for more information.",
    6202: "You cannot modify the options under [Privilege], [Owner] or [Group] because the file is located on a disk formatted in FAT or NTFS.",
    6203: "You are not allowed to modify the file's access privileges because it is within a remote folder.",
    6204: "The selected folder is read-only.",
    6205: "Sorry, this user account has not been assigned proper privileges to perform this action.",
    6206: "File or directory does not exist.",
    6207: "File or directory does not exist.",
    6208: "Cannot find the specified domain. Please contact the system administrator.",
    6209: "Sorry, this user account has not been assigned proper privileges to perform this action.",
    6210: "The changes you made will not be applied because the administrator account \"{0}\" has full privilege control.",
    6211: "Please specify a valid user or group.",
    6212: "The mounted folder does not support Windows ACL. Please refer to Help for more information.",
    6213: "User disk quota exceeded.",
    6215: "ACL explicit permissions exceeds the maximum number of entries.",
    6216: "Unable to store more ACL entries. Please check the usage and quota of the volume and shared folder to ensure there is sufficient capacity for storing ACL entries.",
    6703: "One of the port numbers is used by another service. Please choose another port range.",
    6704: "One of the port numbers is used by another service. Please choose another port range.",
    6705: "The user doesn't exist.",
    6706: "The user name already exists or is reserved for system use only. Please enter a different one.",
    6708: "Volume not found. Please check if the volume status is normal.",
    6709: "Failed to enable the rsync backup service due to insufficient storage space. Please check your volume status and try again.",
    7100: "Failed to deactivate the user. There should be at least one valid DSM administrator (admin or a user belonging to the\u00a0 administrators group) in the system.",
    8000: "WiFi daemon not ready, please wait...",
    8001: "Network daemon not ready, please wait ...",
    8002: "USB Modem daemon not ready, please wait ...",
    8003: "The max characters have been reached.",
    8010: "Failed to get network topology.",
    8011: "Failed to get network information.",
    8012: "Failed to get network settings.",
    8013: "Failed to get WiFi settings.",
    8020: "Failed to set network topology.",
    8021: "Failed to set network.",
    8022: "Failed to configure network settings.",
    8023: "Failed to configure WiFi settings.",
    8030: "Failed to get redirection information.",
    8031: "The DHCP range of interfaces {0} and {1} are in conflict.",
    8100: "Failed to get guest network countdown.",
    8101: "Failed to set guest network countdown.",
    8130: "Failed to get PPPoE settings.",
    8131: "Failed to set PPPoE settings.",
    8132: "No PPPoE interface is availabe.",
    8133: "Failed to start PPPoE service.",
    8134: "Failed to stop PPPoE service.",
    8135: "PPPoE connection failed",
    8136: "PPPoE disconnection failed.",
    8150: "Failed to get geocode.",
    8151: "Failed to set geocode.",
    8152: "Failed to read geocode list.",
    8153: "Geocode does not support this region.",
    8170: "The maximum number ({0}) of rules allowed for the router has been exceeded. Please remove some rules and try again.",
    8175: "",
    8180: "The maximum number ({0}) of rules allowed for the router has been exceeded. Please remove some rules and try again.",
    8190: "",
    8200: "",
    8230: "",
    8231: "",
    9060: "The format of the uploaded pack may be corrupted or incorrect. Please go to Synology's <a class=\"link-font\" href=\"https://www.synology.com/support/download\" target=\"_blank\">Download Center</a> and download the Offline Update Pack again.",
    9061: "The available drive space of this system is insufficient.",
    9062: "The uploaded pack contains a drive database version older than the current one. Please go to Synology's <a class=\"link-font\" href=\"https://www.synology.com/support/download\" target=\"_blank\">Download Center</a> and download the latest Offline Update Pack.",
    9063: "The update pack is incompatible with the system. Please go to Synology's <a class=\"link-font\" href=\"https://www.synology.com/support/download\" target=\"_blank\">Download Center</a> and download the correct Offline Update Pack for your model."
})